package speiger.src.coreengine;

import java.io.File;
import java.nio.ByteBuffer;
import java.nio.file.Paths;

import javax.imageio.ImageIO;

import org.lwjgl.glfw.GLFW;
import org.lwjgl.stb.STBImage;

import speiger.src.coreengine.assets.parsers.NativeMemoryParser;
import speiger.src.coreengine.math.misc.ColorSpaces;
import speiger.src.coreengine.math.misc.ColorUtils;
import speiger.src.coreengine.rendering.input.devices.FileDrop;
import speiger.src.coreengine.rendering.input.devices.Joystick;
import speiger.src.coreengine.rendering.input.devices.Keyboard;
import speiger.src.coreengine.rendering.input.devices.Mouse;
import speiger.src.coreengine.rendering.input.window.Window;
import speiger.src.coreengine.rendering.input.window.WindowManager;
import speiger.src.coreengine.utils.eventbus.EventBus;

public class NewInputTest {
	
	EventBus bus = new EventBus();
	WindowManager manager = new WindowManager();
	
	public static void main(String[] args) {
		new NewInputTest().run();
	}
	
	public void run() {
		GLFW.glfwInit();
		manager.initialize();
		Mouse.INSTANCE.init(bus);
		Keyboard.INSTANCE.init(bus);
		Joystick.INSTANCE.init(manager, bus);
		FileDrop.INSTANCE.init(bus);
		manager.addDevices(Mouse.INSTANCE, Keyboard.INSTANCE, Joystick.INSTANCE, FileDrop.INSTANCE);
		Window window = manager.builder().title("Testing Engine").build();
		testInit();
		window.visible(true);
		while(!window.shouldClose()) {
			GLFW.glfwPollEvents();
			window.beginFrame();
			window.handleInput();
			window.finishFrame();
			try { Thread.sleep(100); }
			catch(InterruptedException e) { e.printStackTrace(); }
		}
		window.destroy();
		manager.destroy();
	}
	
	public void testInit() {
		try {
			int result = ImageIO.read(new File("test.png")).getRGB(0, 0);
			ByteBuffer buffer = NativeMemoryParser.INSTANCE.parseAsset(Paths.get("test.png"), T -> {});
			int[] width = new int[1];
			int[] height = new int[1];
			int[] fileChannels = new int[1];
			ByteBuffer stbBuffer = STBImage.stbi_load_from_memory(buffer, width, height, fileChannels, 4);
			int stbResult = stbBuffer.getInt();
			System.out.println("Java="+ColorUtils.toHex(result, true)+", STB="+ColorUtils.toHex(stbResult, true)+", STB_RGB="+ColorUtils.toHex(ColorSpaces.ABGR.toARGB(stbResult), true));
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}
